#!/bin/bash -e

#
# This file gives the basic configuration for Trilinos for CUDA on the ATTB machines
#
# NOTE: Must first:
#
#   source source_attb_cuda_env.sh
#

EXPERIMENTAL_MUELU=ON
if [ "$USE_CUDA" == "ON" ]; then 
    EXPERIMENTAL_MUELU=OFF
fi

cmake \
-D Trilinos_ENABLE_EXPLICIT_INSTANTIATION:BOOL=ON \
-D Trilinos_ENABLE_INSTALL_CMAKE_CONFIG_FILES:BOOL=ON \
-D BUILD_SHARED_LIBS:BOOL=OFF \
-D Trilinos_ENABLE_DEBUG=OFF \
-D CMAKE_BUILD_TYPE:STRING=${BUILD_TYPE} \
-D Phalanx_KOKKOS_DEVICE_TYPE:STRING="${NODE_TYPE}" \
-D Trilinos_ENABLE_Fortran:BOOL=ON \
-D HAVE_INTREPID_KOKKOSCORE:BOOL=ON \
-D Panzer_ENABLE_FADTYPE:STRING="Sacado::Fad::DFad<RealType>" \
-D MueLu_ENABLE_Experimental:BOOL=${EXPERIMENTAL_MUELU} \
-D Xpetra_ENABLE_Experimental:BOOL=${EXPERIMENTAL_MUELU} \
-D TPL_ENABLE_GLM=OFF \
-D Panzer_ENABLE_EXPLICIT_INSTANTIATION:BOOL=ON \
-D SEACASExodus_ENABLE_MPI:BOOL=ON \
-D EpetraExt_ENABLE_HDF5:BOOL=OFF \
-D Teuchos_ENABLE_LONG_LONG_INT:BOOL=OFF \
-D Intrepid_ENABLE_DEBUG_INF_CHECK=OFF \
-D IntrepidIntrepid2_ENABLE_DEBUG_INF_CHECK:BOOL=OFF \
-D CMAKE_CXX_COMPILER:FILEPATH="mpicxx" \
-D CMAKE_C_COMPILER:FILEPATH="mpicc" \
-D CMAKE_Fortran_COMPILER:FILEPATH="mpif77" \
-D CMAKE_C_FLAGS:STRING="${EXTRA_FLAGS}" \
-D CMAKE_CXX_FLAGS:STRING="${EXTRA_FLAGS}" \
-D CMAKE_Fortran_FLAGS:STRING="${EXTRA_FLAGS}" \
-D CMAKE_VERBOSE_MAKEFILE:BOOL=OFF \
-D CMAKE_SKIP_RULE_DEPENDENCY=ON \
-D Trilinos_VERBOSE_CONFIGURE:BOOL=OFF \
-D Trilinos_ENABLE_STRONG_CXX_COMPILE_WARNINGS=OFF \
-D Trilinos_ENABLE_STRONG_C_COMPILE_WARNINGS=OFF \
-D Trilinos_ENABLE_SHADOW_WARNINGS=OFF \
-D TPL_ENABLE_MPI:BOOL=ON \
-D MPI_EXEC_POST_NUMPROCS_FLAGS:STRING="${MPI_POST_FLAG}" \
-D TPL_ENABLE_HWLOC:BOOL=${USE_HWLOC} \
-D TPL_ENABLE_Boost:BOOL=ON \
-D TPL_ENABLE_Netcdf:BOOL=ON \
-D Netcdf_INCLUDE_DIRS:FILEPATH="${NETCDF_ROOT}/include" \
-D HDF5_INCLUDE_DIRS:FILEPATH="${HDF5_ROOT}/include" \
-D Netcdf_LIBRARY_DIRS:FILEPATH="${NETCDF_ROOT}/lib" \
-D HDF5_LIBRARY_DIRS:FILEPATH="${HDF5_ROOT}/lib" \
-D TPL_HDF5_LIBRARIES:FILEPATH="-L${HDF5_ROOT}/lib;${HDF5_ROOT}/lib/libhdf5_hl.a;${HDF5_ROOT}/lib/libhdf5.a;-lz;-ldl" \
-D TPL_Netcdf_LIBRARIES="-L${BOOST_ROOT}/lib;-L${NETCDF_ROOT}/lib;-L${NETCDF_ROOT}/lib;-L${PNETCDF_ROOT}/lib;-L${HDF5_ROOT}/lib;${BOOST_ROOT}/lib/libboost_program_options.a;${BOOST_ROOT}/lib/libboost_system.a;${NETCDF_ROOT}/lib/libnetcdf.a;${PNETCDF_ROOT}/lib/libpnetcdf.a;${HDF5_ROOT}/lib/libhdf5_hl.a;${HDF5_ROOT}/lib/libhdf5.a;-lz;-ldl" \
-D TPL_ENABLE_BoostLib=ON \
-DBoostLib_LIBRARY_DIRS=${BOOST_ROOT}/lib \
-D TPL_ENABLE_Netcdf:BOOL=ON \
-D TPL_ENABLE_HDF5:BOOL=ON \
-D TPL_ENABLE_Matio=OFF  \
-D TPL_ENABLE_LAPACK:BOOL=ON \
-D TPL_LAPACK_LIBRARIES:FILEPATH="${LAPACK_LIB}" \
-D TPL_ENABLE_BLAS:BOOL=ON \
-D TPL_BLAS_LIBRARIES:FILEPATH="${LAPACK_LIB}" \
-D TPL_HWLOC_LIBRARIES:FILEPATH="${HWLOC_LIBS}" \
-D CMAKE_SKIP_RULE_DEPENDENCY=ON \
-D TPL_ENABLE_Matio:BOOL=OFF \
-D TPL_ENABLE_X11:BOOL=OFF \
-D TPL_ENABLE_SuperLU:BOOL=OFF \
-D Trilinos_ENABLE_OpenMP=${USE_OPENMP} \
-D Kokkos_ENABLE_OpenMP:BOOL=${USE_OPENMP} \
-D Kokkos_ENABLE_Pthread:BOOL=${USE_PTHREADS} \
-D TPL_ENABLE_CUDA:BOOL=${USE_CUDA} \
-D TPL_ENABLE_CUSPARSE:BOOL=${USE_CUDA} \
-D Kokkos_ENABLE_Cuda_UVM:BOOL=${USE_CUDA} \
-D Kokkos_ENABLE_Debug_Bounds_Check=ON \
"$@" \
${TRILINOS_BASE_DIR}/Trilinos
